const parseString = require('xml2js').parseString;
const fs = require('fs');

const Report = require('../lib/report.js');

function callstack_print(callstack) {
  let buffer = '';

  /* Compute format string according to longest function name and line number. */
  for (var i = 0; i < callstack.length; i++) {
    var frame = callstack[i];

    buffer += `#${i} ${frame.ip ? frame.ip[0] : '--'} in ${
      frame.fn ? frame.fn[0] : '--'
    } at ${frame.file ? frame.file[0] : '--'} ${
      frame.line ? frame.line[0] : '--'
    } ${frame.obj ? frame.obj[0] : '--'}`;
  }

  return buffer;
}

class valgrind {
  constructor(options) {
    this.options = options;
    this.file = options.file;
    this.data = options.data;
    this.exclude = options.exclude;
    if (!this.exclude) {
      this.exclude = [];
    }
  }

  extractError(p, data, root) {
    var self = this;

    /* If exclusions are set up, take them into account. */
    if (self.exclude.length > 0 && data.kind) {
      var kind = data.kind[0];

      if (kind.indexOf('Leak') > -1 && self.exclude.indexOf('leaks') > -1) {
        return;
      }
    }

    /* Inherit any properties from the parent report. */
    var report = new Report(p);
    if (data.kind) {
      var kind = data.kind;

      for (var i = 0; i < kind.length; i++) {
        report.add('classifiers', kind[i]);
      }
    }

    var thread = report.thread(data.tid[0]);
    var what = data.what ? data.what[0] : null;

    if (data.xwhat) {
      var xw = data.xwhat[0];

      if (xw.leakedbytes)
        report.set('attribute', 'leaked.bytes', xw.leakedbytes[0]);

      if (xw.leakedblocks)
        report.set('attribute', 'leaked.blocks', xw.leakedblocks[0]);

      if (!what) {
        what = xw.text[0];
      }
    }

    /* Push the faulting frames. */
    var faulting_frame;

    for (var i = 0; i < data.stack[0].frame.length; i++) {
      var frame = data.stack[0].frame[i];
      var p = {};

      if (!faulting_frame) {
        faulting_frame = p;
      }

      if (frame.ip) {
        p.registers = {};
        p.registers.ip = frame.ip[0];
      }

      if (frame.fn) {
        p['function'] = frame.fn[0];
      }

      if (frame.line) {
        p['line'] = frame.line[0];
      }

      if (frame.obj) {
        p['object'] = frame.obj[0];
      }

      if (frame.file) {
        p['file'] = frame.file[0];
      }

      if (frame.file && frame.dir)
        p['path'] = frame.dir[0] + '/' + frame.file[0];

      thread.frame(p);
    }

    if (faulting_frame) faulting_frame.text = what + '\n\n';

    /*
     * Extract potential call sites and more. Currently, these are serialized
     * as a source code text buffer. Finally, if there is auxiliary data attempt
     * to extract it.
     */
    if (data.auxwhat && data.stack.length > 1) {
      var aw = data.auxwhat;
      var buffer = '';

      for (var i = 0; i < aw.length && i < data.stack.length - 1; i++) {
        var callstack = data.stack[i + 1].frame;

        buffer += aw[i] + '\n';
        buffer += callstack_print(callstack);
        buffer += '\n';
      }

      if (!faulting_frame.text) faulting_frame.text = '';

      faulting_frame.text += buffer;
    } else if (data.auxwhat) {
      /* There is auxiliary data, associate it with the faulting frame. */
      var aw = data.auxwhat;
      var buffer = '';

      for (var i = 0; i < aw.length; i++) {
        buffer += aw[i] + '\n';
      }

      if (!faulting_frame.text) faulting_frame.text = '';

      faulting_frame.text += buffer;
    }
  }

  extract(options, data, raw, cb) {
    var self = this;
    var report = new Report();

    data = data.valgrindoutput;

    var cursor = data.preamble;
    var buffer = '';

    if (data.tool[0] !== 'memcheck')
      throw new Error('unsupported tool: ' + data.tool[0]);

    for (var i = 0; i < cursor.length; i++) {
      if (cursor[i].line) {
        for (var j = 0; j < cursor[i].line.length; j++) {
          var line = cursor[i].line[j];

          buffer += line + '\n';
        }
      }
    }

    report.checksum(raw);
    if (options && options.attributes) {
      for (var a in options.attributes)
        report.set('attribute', a, options.attributes[a]);
    }

    if (data.pid) {
      report.set('attribute', 'pid', data.pid[0]);
    }

    if (data.ppid) {
      report.set('attribute', 'ppid', data.ppid[0]);
    }

    report
      .set('annotation', { Preamble: buffer })
      .set('attribute', 'tool', data.tool[0])
      .add('classifiers', data.tool[0]);

    if (data.args && data.args.length > 0) {
      var args = data.args[0];

      if (args.vargv) {
        var varg = args.vargv[0];
        var buffer = '';

        if (varg.exe) {
          for (var i = 0; i < varg.exe.length; i++) {
            buffer += varg.exe[i] + ' ';
          }
        }

        if (varg.arg) {
          for (var i = 0; i < varg.arg.length; i++) {
            buffer += varg.arg[i] + ' ';
          }
        }

        buffer = buffer.substring(0, buffer.length - 1);
        report.set('annotation', { 'Valgrind Command': buffer });
      }

      if (args.argv) {
        var arg = args.argv[0];
        var buffer = '';
        var application;

        if (arg.exe) {
          for (var i = 0; i < arg.exe.length; i++) {
            buffer += arg.exe[i] + ' ';
          }
        }

        if (buffer !== '' && buffer !== ' ') {
          var application = buffer.substring(buffer, buffer.length - 1);
          report.set('attribute', 'application', application);
        }

        if (arg.arg) {
          for (var i = 0; i < arg.arg.length; i++) {
            buffer += arg.arg[i] + ' ';
          }
        }

        buffer = buffer.substring(0, buffer.length - 1);
        report.set('annotation', { Command: buffer });
      }
    }

    /*
     * If status objects exist, then fold them into the global annotation
     * list.
     */
    if (data.status) {
      var k = 0;
      var o = {};

      for (var j = 0; j < data.status.length; j++) {
        var s = data.status[j];
        o[s.state[0] + ' (' + k++ + ')'] = s.time[0];
      }

      report.set('annotation', { Status: o });
    }

    if (data.error) {
      for (var i = 0; i < data.error.length; i++) {
        self.extractError(report, data.error[i], data);
      }
    }

    return cb(null, report);
  }

  parse(options, cb) {
    var self = this;

    if (self.data) {
      parseString(self.data, function (e, r) {
        if (e) {
          cb(e, r);
        }
        if (!r) {
          return cb(new Error('Unsupported XML protocol version'));
        }
        if (!r.valgrindoutput || r.valgrindoutput.protocolversion[0] !== '4') {
          return cb(new Error('Unsupported XML protocol version'));
        }

        return self.extract(options, r, self.data, cb);
      });
    } else if (self.file) {
      var data = fs.readFile(this.file, function (e, d) {
        if (e) return cb(e, r);

        parseString(d, function (e, r) {
          if (e) return cb(e, r);

          if (
            !r.valgrindoutput ||
            r.valgrindoutput.protocolversion[0] !== '4'
          ) {
            return cb(new Error('Unsupported XML protocol version'));
          }

          return self.extract(options, r, d, cb);
        });
      });
    } else {
      return cb('no data was supplied');
    }
  }
}

module.exports = valgrind;

//-- vim:ts=2:et:sw=2:st=2
