"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.anySignal = exports.createAbortController = exports.AbortController = exports.AbortSignal = void 0;
const AbortError_1 = require("./AbortError");
const Events_1 = require("./Events");
const abortInterfaces_1 = require("./abortInterfaces");
/**
 * Copied and repurposed from https://github.com/mo/abortcontroller-polyfill/blob/master/src/abortcontroller.js
 */
class Emitter {
    constructor() {
        this._listeners = {};
        this._events = new Events_1.Events();
    }
    addEventListener(type, listener, options) {
        if (!(type in this._listeners)) {
            this._listeners[type] = [];
        }
        const callback = (ev) => {
            if (typeof listener === 'object') {
                return listener.handleEvent.call(this, ev);
            }
            else {
                return listener.call(this, ev);
            }
        };
        const { once, signal } = (typeof options === 'object' ? options : {});
        if (once) {
            this._events.once(type, callback);
        }
        else {
            this._events.on(type, callback);
        }
        if (signal) {
            const removeFn = () => this.removeEventListener(type, listener);
            signal.addEventListener(type, removeFn, { once: true });
        }
        this._listeners[type].push({ callback, listener, options });
    }
    removeEventListener(type, listener) {
        if (!(type in this._listeners)) {
            return;
        }
        const allListeners = this._listeners[type];
        const listeners = allListeners.filter((l) => l.listener === listener);
        for (const elem of listeners) {
            this._events.off(type, elem.callback);
            const index = allListeners.indexOf(elem);
            allListeners.splice(index, 1);
        }
    }
    dispatchEvent(event) {
        this._events.emit(event.type, event);
        return !event.defaultPrevented;
    }
}
/**
 * Copied and repurposed from https://github.com/mo/abortcontroller-polyfill/blob/master/src/abortcontroller.js
 */
class AbortSignal extends Emitter {
    constructor() {
        super();
        this.aborted = false;
        this.onabort = null;
        // Compared to assignment, Object.defineProperty makes properties non-enumerable by default and
        // we want Object.keys(new AbortController().signal) to be [] for compat with the native impl
        Object.defineProperty(this, 'aborted', { writable: true, configurable: true, enumerable: false });
        Object.defineProperty(this, 'onabort', { writable: true, configurable: true, enumerable: false });
        Object.defineProperty(this, 'reason', { writable: true, configurable: true, enumerable: false });
    }
    toString() {
        return '[object AbortSignal]';
    }
    throwIfAborted() {
        if (this.aborted) {
            throw this.reason;
        }
    }
    dispatchEvent(event) {
        if (event.type === 'abort') {
            this.aborted = true;
            if (typeof this.onabort === 'function') {
                this.onabort.call(this, event);
            }
        }
        return super.dispatchEvent(event);
    }
}
exports.AbortSignal = AbortSignal;
/**
 * Copied and repurposed from https://github.com/mo/abortcontroller-polyfill/blob/master/src/abortcontroller.js
 */
class AbortController {
    constructor() {
        // Compared to assignment, Object.defineProperty makes properties non-enumerable by default and
        // we want Object.keys(new AbortController()) to be [] for compat with the native impl
        this.signal = new AbortSignal();
        Object.defineProperty(this, 'signal', { configurable: true, enumerable: false });
    }
    abort(reason) {
        let event;
        try {
            event = new Event('abort');
        }
        catch (e) {
            if (typeof document !== 'undefined') {
                const ie8Document = document;
                if (!ie8Document.createEvent && ie8Document.createEventObject) {
                    // For Internet Explorer 8:
                    event = ie8Document.createEventObject();
                    event.type = 'abort';
                }
                else {
                    // For Internet Explorer 11:
                    event = document.createEvent('Event');
                    event.initEvent('abort', false, false);
                }
            }
            else {
                // Fallback where document isn't available:
                event = {
                    type: 'abort',
                    bubbles: false,
                    cancelable: false,
                };
            }
        }
        let signalReason = reason;
        if (signalReason === undefined) {
            if (typeof document === 'undefined') {
                signalReason = new AbortError_1.AbortError('This operation was aborted');
            }
            else {
                try {
                    signalReason = new DOMException('signal is aborted without reason');
                }
                catch (err) {
                    // IE 11 does not support calling the DOMException constructor, use a
                    // regular error object on it instead.
                    signalReason = new AbortError_1.AbortError('This operation was aborted');
                }
            }
        }
        this.signal.reason = signalReason;
        this.signal.dispatchEvent(event);
    }
    toString() {
        return '[object AbortController]';
    }
}
exports.AbortController = AbortController;
/**
 * Copied and repurposed from https://github.com/mo/abortcontroller-polyfill/blob/master/src/abortcontroller.js
 */
if (typeof Symbol !== 'undefined' && Symbol.toStringTag) {
    // These are necessary to make sure that we get correct output for:
    // Object.prototype.toString.call(new AbortController())
    AbortController.prototype[Symbol.toStringTag] = 'AbortController';
    AbortSignal.prototype[Symbol.toStringTag] = 'AbortSignal';
}
/**
 * Creates a new abort controller.
 *
 * If the AbortController is not available, a polyfill is used.
 * @returns
 */
function createAbortController() {
    if (abortInterfaces_1.OriginalAbortController) {
        return new abortInterfaces_1.OriginalAbortController();
    }
    else {
        return new AbortController();
    }
}
exports.createAbortController = createAbortController;
function anySignal(...signals) {
    const controller = createAbortController();
    function onAbort() {
        controller.abort();
        // Cleanup
        for (const signal of signals) {
            if (signal) {
                signal.removeEventListener('abort', onAbort);
            }
        }
    }
    for (const signal of signals) {
        if (!signal) {
            continue;
        }
        if (signal.aborted) {
            onAbort();
            break;
        }
        signal.addEventListener('abort', onAbort);
    }
    return controller.signal;
}
exports.anySignal = anySignal;
//# sourceMappingURL=AbortController.js.map