"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BreadcrumbsManager = void 0;
const _1 = require(".");
const jsonEscaper_1 = require("../../common/jsonEscaper");
const ConsoleEventSubscriber_1 = require("./events/ConsoleEventSubscriber");
const InMemoryBreadcrumbsStorage_1 = require("./storage/InMemoryBreadcrumbsStorage");
const BREADCRUMB_ATTRIBUTE_NAME = 'breadcrumbs.lastId';
class BreadcrumbsManager {
    constructor(configuration, options) {
        var _a, _b, _c;
        /**
         * Determines if the breadcrumb manager is enabled.
         */
        this._enabled = false;
        this._eventSubscribers = [new ConsoleEventSubscriber_1.ConsoleEventSubscriber()];
        this.breadcrumbsType = (_a = configuration === null || configuration === void 0 ? void 0 : configuration.eventType) !== null && _a !== void 0 ? _a : _1.defaultBreadcurmbType;
        this.logLevel = (_b = configuration === null || configuration === void 0 ? void 0 : configuration.logLevel) !== null && _b !== void 0 ? _b : _1.defaultBreadcrumbsLogLevel;
        this._storage = (_c = options === null || options === void 0 ? void 0 : options.storage) !== null && _c !== void 0 ? _c : new InMemoryBreadcrumbsStorage_1.InMemoryBreadcrumbsStorage(configuration === null || configuration === void 0 ? void 0 : configuration.maximumBreadcrumbs);
        this._interceptor = configuration === null || configuration === void 0 ? void 0 : configuration.intercept;
        if (options === null || options === void 0 ? void 0 : options.subscribers) {
            this._eventSubscribers.push(...options.subscribers);
        }
    }
    addEventSubscriber(subscriber) {
        if (this._enabled) {
            subscriber.start(this);
        }
        this._eventSubscribers.push(subscriber);
    }
    setStorage(storage) {
        this._storage = storage;
    }
    dispose() {
        this._enabled = false;
        for (const subscriber of this._eventSubscribers) {
            subscriber.dispose();
        }
    }
    bind({ client, reportEvents }) {
        for (const attachment of this._storage.getAttachments()) {
            client.addAttachment(attachment);
        }
        client.addAttribute(() => ({
            [BREADCRUMB_ATTRIBUTE_NAME]: this._storage.lastBreadcrumbId,
        }));
        reportEvents.on('before-skip', (report) => this.logReport(report));
    }
    initialize() {
        if (this._enabled) {
            return;
        }
        for (const subscriber of this._eventSubscribers) {
            subscriber.start(this);
        }
        this._enabled = true;
    }
    verbose(message, attributes) {
        return this.log(message, _1.BreadcrumbLogLevel.Verbose, attributes);
    }
    debug(message, attributes) {
        return this.log(message, _1.BreadcrumbLogLevel.Debug, attributes);
    }
    info(message, attributes) {
        return this.log(message, _1.BreadcrumbLogLevel.Info, attributes);
    }
    warn(message, attributes) {
        return this.log(message, _1.BreadcrumbLogLevel.Warning, attributes);
    }
    error(message, attributes) {
        return this.log(message, _1.BreadcrumbLogLevel.Error, attributes);
    }
    log(message, level, attributes) {
        return this.addBreadcrumb(message, level, _1.BreadcrumbType.Manual, attributes);
    }
    logReport(report) {
        const level = report.data instanceof Error ? _1.BreadcrumbLogLevel.Error : _1.BreadcrumbLogLevel.Warning;
        return this.addBreadcrumb(report.message, level, _1.BreadcrumbType.System);
    }
    addBreadcrumb(message, level, type, attributes) {
        if (!this._enabled) {
            return false;
        }
        let rawBreadcrumb = {
            message: this.prepareBreadcrumbMessage(message),
            level,
            type,
            attributes,
        };
        if (this._interceptor) {
            const interceptorBreadcrumb = this._interceptor(rawBreadcrumb);
            if (!interceptorBreadcrumb) {
                return false;
            }
            rawBreadcrumb = interceptorBreadcrumb;
        }
        if ((this.logLevel & rawBreadcrumb.level) !== level) {
            return false;
        }
        if ((this.breadcrumbsType & rawBreadcrumb.type) !== type) {
            return false;
        }
        this._storage.add(rawBreadcrumb);
        return true;
    }
    /**
     * The expectation is, message should always be defined and passed as string.
     * However, logger can pass as a message an object or any other unknown type.
     * To be sure the code won't break, this method ensures the message is always a string
     * no matter what the logger gives us.
     * @param message breadcrumb message
     */
    prepareBreadcrumbMessage(message) {
        if (message == null) {
            return '';
        }
        const messageType = typeof message;
        switch (messageType) {
            case 'string': {
                return message;
            }
            case 'object': {
                return JSON.stringify(message, (0, jsonEscaper_1.jsonEscaper)());
            }
            default: {
                return message.toString();
            }
        }
    }
}
exports.BreadcrumbsManager = BreadcrumbsManager;
//# sourceMappingURL=BreadcrumbsManager.js.map