"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V8StackTraceConverter = void 0;
const frameNamesConsts_1 = require("./consts/frameNamesConsts");
class V8StackTraceConverter {
    get engine() {
        return 'v8';
    }
    constructor(addressSeparator = '') {
        this.addressSeparator = addressSeparator;
    }
    convert(stackTrace, message) {
        const result = [];
        let stackFrames = stackTrace.split('\n');
        const errorHeader = message.split('\n');
        // remove error header from stack trace - if the error header exists
        if (stackFrames[0].indexOf(errorHeader[0]) !== -1) {
            stackFrames = stackFrames.slice(errorHeader.length);
        }
        else {
            stackFrames = stackFrames.slice(1);
        }
        for (const stackFrame of stackFrames) {
            const normalizedStackFrame = stackFrame.trim();
            if (!normalizedStackFrame) {
                continue;
            }
            const frame = this.parseFrame(normalizedStackFrame);
            result.push(frame);
        }
        return result;
    }
    parseFrame(stackFrame) {
        const frameSeparator = 'at ';
        if (!stackFrame.startsWith(frameSeparator)) {
            return {
                funcName: stackFrame,
                library: frameNamesConsts_1.UNKNOWN_FRAME,
            };
        }
        stackFrame = stackFrame.substring(stackFrame.indexOf(frameSeparator) + frameSeparator.length);
        const sourceCodeSeparator = ' (';
        const sourceCodeStartIndex = stackFrame.indexOf(sourceCodeSeparator);
        const anonymousFunction = sourceCodeStartIndex === -1;
        if (anonymousFunction) {
            return Object.assign({ funcName: frameNamesConsts_1.ANONYMOUS_FUNCTION }, this.parseSourceCodeInformation(stackFrame));
        }
        return Object.assign({ funcName: stackFrame.substring(0, sourceCodeStartIndex) }, this.parseSourceCodeInformation(stackFrame.substring(sourceCodeStartIndex + sourceCodeSeparator.length, stackFrame.length - 1)));
    }
    parseSourceCodeInformation(sourceCodeInformation) {
        if (sourceCodeInformation.startsWith('eval')) {
            return this.extractEvalInformation(sourceCodeInformation);
        }
        if (this.addressSeparator && sourceCodeInformation.startsWith(this.addressSeparator)) {
            sourceCodeInformation = sourceCodeInformation.substring(this.addressSeparator.length).trimStart();
        }
        const sourceCodeParts = sourceCodeInformation.split(':');
        const column = parseInt(sourceCodeParts[sourceCodeParts.length - 1]);
        const lineNumber = parseInt(sourceCodeParts[sourceCodeParts.length - 2]);
        const library = sourceCodeParts.slice(0, sourceCodeParts.length - 2).join(':');
        return {
            library,
            column: isNaN(column) ? undefined : column,
            line: isNaN(lineNumber) ? undefined : lineNumber,
        };
    }
    extractEvalInformation(evalSourceCodeInformation) {
        const sourceCodeStartSeparatorChar = '(';
        const sourceCodeEndSeparatorChar = ')';
        const sourceCodeStart = evalSourceCodeInformation.indexOf(sourceCodeStartSeparatorChar);
        const sourceCodeEnd = evalSourceCodeInformation.indexOf(sourceCodeEndSeparatorChar);
        if (sourceCodeStart === -1 || sourceCodeEnd === -1 || sourceCodeStart > sourceCodeEnd) {
            return {
                library: frameNamesConsts_1.UNKNOWN_FRAME,
            };
        }
        const sourceCodeInformation = evalSourceCodeInformation.substring(sourceCodeStart + sourceCodeStartSeparatorChar.length, sourceCodeEnd);
        return this.parseSourceCodeInformation(sourceCodeInformation);
    }
}
exports.V8StackTraceConverter = V8StackTraceConverter;
//# sourceMappingURL=V8StackTraceConverter.js.map